@@ -25,7 +25,7 @@
 #' `.mse_sample_apply` that does **only** split the `Spectra` and no other data
 #' the `MsExperiment`.
 #'
-#' This function splits the spectra by x@sampleDataLinks [, 1] which is ALWAYS
+#' This function splits the spectra by `x@sampleDataLinks[, 1]` which is ALWAYS
 #' ordered from 1 to number of samples. The result will thus be a `list` in the
 #' same order (sample 1:length(x)).
 #'
@@ -63,7 +63,8 @@
 .mse_find_chrom_peaks_sample <- function(x, msLevel = 1L, param, ...) {
     x <- filterMsLevel(x, msLevel)
     pkd <- Spectra::peaksData(x, columns = c("mz", "intensity"),
-                              f = factor(), BPPARAM = SerialParam())
+                              f = factor(), return.type = "list",
+                              BPPARAM = SerialParam())
     vals_per_spect <- vapply(pkd, nrow, integer(1), USE.NAMES = FALSE)
     ## Open questions:
     ## - What to do with empty spectra? Remove them? MatchFilter does not like
@@ -116,6 +117,8 @@
 #' (as a new spectra variable). This can be used by FUN to split the spectra
 #' by sample and process them separately (and in parallel).
 #'
+#' @param x `MsExperiment`
+#'
 #' @author Johannes Rainer
 #'
 #' @noRd
@@ -139,9 +142,9 @@
     }
     sps <- spectra(x)[x@sampleDataLinks[["spectra"]][, 2L]]
     sps$.SAMPLE_IDX <- x@sampleDataLinks[["spectra"]][, 1L] # or as.factor?
-    lapply(chunks, function(z, ..., pb) {
+    lapply(chunks, function(z, ..., BPPARAM, pb) {
         suppressMessages(
-            res <- FUN(sps[sps$.SAMPLE_IDX %in% z], ...)
+            res <- FUN(sps[sps$.SAMPLE_IDX %in% z], ..., BPPARAM = BPPARAM)
         )
         if (progressbar) pb$tick()
         res
@@ -230,7 +233,7 @@
     }
     bpmapply(
         split(peaksData(x, columns = c("mz", "intensity"), f = factor(),
-                        BPPARAM = SerialParam()), f),
+                        return.type = "list", BPPARAM = SerialParam()), f),
         split(rtime(x), f),
         FUN = function(p, rt, prm, msl) {
             vals_per_spect <- vapply(p, nrow, integer(1), USE.NAMES = FALSE)
@@ -280,7 +283,7 @@
     else f <- factor(integer(), levels = sidx)
     bplapply(
         split(Spectra::peaksData(x, columns = c("mz", "intensity"),
-                                 f = factor(),
+                                 f = factor(), return.type = "list",
                                  BPPARAM = SerialParam()), f),
         FUN = .peaksdata_profmat, method = method, step = step,
         baselevel = baselevel, basespace = basespace, mzrange. = mzrange.,
@@ -345,7 +348,8 @@
     if (!(ref_idx %in% seq_along(x)))
         stop("'centerSample' needs to be an integer between 1 and ", length(x))
     ref_sps <- filterMsLevel(spectra(x[ref_idx]), msLevel = msLevel)
-    ref_pm <- .peaksdata_profmat(peaksData(ref_sps, f = factor()),
+    ref_pm <- .peaksdata_profmat(peaksData(ref_sps, f = factor(),
+                                           return.type = "list"),
                                  method = "bin", step = binSize(param),
                                  returnBreaks = TRUE)
     res <- unlist(.mse_spectrapply_chunks(
@@ -388,11 +392,13 @@
     if (!(length(ref) & length(other)))
         stop("No spectra with MS level ", msLevel, " present")
     if (!length(ref_pm))
-        ref_pm <- .peaksdata_profmat(peaksData(ref, f = factor()),
+        ref_pm <- .peaksdata_profmat(peaksData(ref, f = factor(),
+                                               return.type = "list"),
                                      method = "bin",
                                      step = binSize(param),
                                      returnBreaks = TRUE)
-    other_pm <- .peaksdata_profmat(peaksData(other, f = factor()),
+    other_pm <- .peaksdata_profmat(peaksData(other, f = factor(),
+                                             return.type = "list"),
                                    method = "bin",
                                    step = binSize(param),
                                    returnBreaks = TRUE)
@@ -458,6 +464,9 @@
     if (is.matrix(mz) && ncol(mz) != 2)
         stop("'mz' is expected to be a two-column matrix", call. = FALSE)
     pks <- cbind(mz, rt)
+    if (anyNA(pks))
+        stop("Missing values (`NA`) in 'rt' or 'mz' are not allowed.",
+             call. = FALSE)
     npks <- nrow(pks)
     if (length(msLevel) != npks)
         msLevel <- rep(msLevel[1L], npks)
@@ -467,20 +476,22 @@
         stop("Length of 'isolationWindow' (if provided) should match the ",
              "number of chromatograms to extract.")
     colnames(pks) <- c("mzmin", "mzmax", "rtmin", "rtmax")
+    rtr <- as.vector(do.call(rbind, reduce(rt[, 1L], rt[, 2L])))
     res <- .mse_spectrapply_chunks(
-        x, FUN = function(z, pks, msl, afun, BPPARAM) {
+        x, FUN = function(z, pks, msl, afun, rtr, BPPARAM) {
             sidx <- unique(z$.SAMPLE_IDX)
             z <- filterMsLevel(z, msLevel = msLevel)
-            rtr <- range(pks[, c("rtmin", "rtmax")], na.rm = TRUE)
-            if (all(is.finite(rtr)))
-                z <- filterRt(z, rt = rtr)
+            if (length(rtr)) # for lower memory footprint
+                z <- filterRanges(
+                    z, ranges = rtr, match = "any",
+                    spectraVariables = rep("rtime", length(rtr) / 2))
             lz <- length(z)
             if (lz)
                 f <- factor(z$.SAMPLE_IDX, levels = sidx)
             else f <- factor(integer(), levels = sidx)
             bpmapply(
                 split(Spectra::peaksData(z, columns = c("mz", "intensity"),
-                                         f = factor(),
+                                         f = factor(), return.type = "list",
                                          BPPARAM = SerialParam()), f),
                 split(rtime(z), f),
                 split(msLevel(z), f),
@@ -491,19 +502,20 @@
                                 pks_tmz = isolationWindow,
                                 aggregationFun = afun),
                 SIMPLIFY = FALSE, USE.NAMES = FALSE, BPPARAM = BPPARAM)
-        }, pks = pks, msl = msLevel, afun = aggregationFun,
+        }, pks = pks, msl = msLevel, afun = aggregationFun, rtr = rtr,
         chunkSize = chunkSize, progressbar = progressbar, BPPARAM = BPPARAM)
-    res <- as(do.call(cbind, unlist(res, recursive = FALSE, use.names = FALSE)),
-              "MChromatograms")
+    res <- do.call(cbind, unlist(res, recursive = FALSE, use.names = FALSE))
     fd <- annotatedDataFrameFrom(res, byrow = TRUE)
     fd$mzmin <- mz[, 1]
     fd$mzmax <- mz[, 2]
     fd$rtmin <- rt[, 1]
     fd$rtmax <- rt[, 2]
-    res@featureData <- fd
-    rownames(res@.Data) <- rownames(fd)
-    res@phenoData <- AnnotatedDataFrame(as.data.frame(sampleData(x)))
-    colnames(res@.Data) <- rownames(pData(res))
+    pd <- AnnotatedDataFrame(as.data.frame(sampleData(x)))
+    rownames(res) <- rownames(fd)
+    colnames(res) <- rownames(pd)
+    res <- as(res, "MChromatograms")
+    slot(res, "featureData", check = FALSE) <- fd
+    slot(res, "phenoData", check = FALSE) <- pd
     res
 }
 
