@@ -5,9 +5,11 @@
 #' Takes a XCMSnExp and drops ProcessHistory steps from the
 #' `@.processHistory` slot matching the provided type.
 #'
-#' @param num which should be dropped? If \code{-1} all matching will be dropped,
+#' @param num which should be dropped? If `-1` all matching will be dropped,
 #'     otherwise just the most recent num.
 #'
+#' @md
+#'
 #' @return The XCMSnExp input object with selected ProcessHistory steps dropped.
 #'
 #' @noRd
@@ -133,31 +135,31 @@ dropGenericProcessHistory <- function(x, fun) {
 
 #' @description
 #'
-#' Extract a \code{data.frame} of retention time, mz and intensity
+#' Extract a `data.frame` of retention time, mz and intensity
 #' values from each file/sample in the provided rt-mz range.
 #'
 #' @note
 #'
-#' Ideally, \code{x} should be an \code{OnDiskMSnExp} object as subsetting
-#' of a \code{XCMSnExp} object is more costly (removing of preprocessing
+#' Ideally, `x` should be an `OnDiskMSnExp` object as subsetting
+#' of a `XCMSnExp` object is more costly (removing of preprocessing
 #' results, restoring data etc). If retention times reported in the
 #' featureData are replaced by adjusted retention times, these are set
 #' in the Spectrum objects as retention time.
 #'
-#' @param x An \code{OnDiskMSnExp} object.
+#' @param x An `OnDiskMSnExp` object.
 #'
-#' @param rt \code{numeric(2)} with the retention time range from which the
+#' @param rt `numeric(2)` with the retention time range from which the
 #'     data should be extracted.
 #'
-#' @param mz \code{numeric(2)} with the mz range.
+#' @param mz `numeric(2)` with the mz range.
 #'
-#' @param msLevel \code{integer} defining the MS level(s) to which the data
+#' @param msLevel `integer` defining the MS level(s) to which the data
 #'     should be restricted prior to data extraction.
 #'
 #' @return
 #'
-#' A \code{list} with length equal to the number of files and
-#' each element being a \code{data.frame} with the extracted values.
+#' A `list` with length equal to the number of files and
+#' each element being a `data.frame` with the extracted values.
 #'
 #' @noRd
 #'
@@ -210,23 +212,23 @@ dropGenericProcessHistory <- function(x, fun) {
 #'
 #' @note This reads the full data first and does the subsetting later in R.
 #'
-#' @param object An \code{XCMSnExp} object representing a single sample.
+#' @param object An `XCMSnExp` object representing a single sample.
 #'
-#' @param peakArea A \code{matrix} with the peak definition, i.e.
-#'     \code{"rtmin"}, \code{"rtmax"}, \code{"mzmin"} and \code{"mzmax"}.
+#' @param peakArea A `matrix` with the peak definition, i.e.
+#'     "rtmin", "rtmax", "mzmin" and "mzmax".
 #'
-#' @param sample_idx \code{integer(1)} with the index of the sample in the
+#' @param sample_idx integer(1) with the index of the sample in the
 #'     object.
 #'
 #' @param mzCenterFun Name of the function to be used to calculate the mz value.
-#'     Defaults to \code{weighted.mean}, i.e. the intensity weighted mean mz.
+#'     Defaults to weighted.mean, i.e. the intensity weighted mean mz.
 #'
-#' @param cn \code{character} with the names of the result matrix.
+#' @param cn character with the names of the result matrix.
 #'
 #' @return
 #'
-#' A \code{matrix} with at least columns \code{"mz"}, \code{"rt"},
-#' \code{"into"} and \code{"maxo"} with the by intensity weighted mean of
+#' A matrix with at least columns `"mz"`, `"rt"`,
+#' `"into"` and `"maxo"` with the by intensity weighted mean of
 #' mz, rt or the maximal intensity in the area, the integrated signal in
 #' the area and the maximal signal in the area.
 #'
@@ -553,7 +555,12 @@ dropGenericProcessHistory <- function(x, fun) {
 }
 
 .hasFilledPeaks <- function(object) {
-    hasChromPeaks(object) & any(chromPeakData(object)$is_filled, na.rm = TRUE)
+    if (is(object, "XcmsExperimentHdf5")) {
+        length(object@gap_peaks_ms_level) > 0
+    } else {
+        hasChromPeaks(object) &
+            any(chromPeakData(object)$is_filled, na.rm = TRUE)
+    }
 }
 
 #' @description
@@ -577,35 +584,6 @@ dropGenericProcessHistory <- function(x, fun) {
     idxs
 }
 
-#' @rdname adjustRtime
-adjustRtimePeakGroups <- function(object, param = PeakGroupsParam(),
-                                  msLevel = 1L) {
-    if (!(inherits(object, "XCMSnExp") | inherits(object, "XcmsExperiment")))
-        stop("'object' has to be an 'XCMSnExp' or 'XcmsExperiment' object.")
-    if (!hasFeatures(object))
-        stop("No features present. Please run 'groupChromPeaks' first.")
-    if (hasAdjustedRtime(object))
-        warning("Alignment/retention time correction was already performed, ",
-                "returning a matrix with adjusted retention times.")
-    subs <- subset(param)
-    if (!length(subs))
-        subs <- seq_along(fileNames(object))
-    nSamples <- length(subs)
-    missingSample <- nSamples - (nSamples * minFraction(param))
-    pkGrp <- .getPeakGroupsRtMatrix(
-        peaks = chromPeaks(object, msLevel = msLevel),
-        peakIndex = .peakIndex(
-            .update_feature_definitions(
-                featureDefinitions(object), rownames(chromPeaks(object)),
-                rownames(chromPeaks(object, msLevel = msLevel)))),
-        sampleIndex = subs,
-        missingSample = missingSample,
-        extraPeaks = extraPeaks(param)
-    )
-    colnames(pkGrp) <- basename(fileNames(object))[subs]
-    pkGrp
-}
-
 .plotChromPeakDensity <- function(object, mz, rt, param, simulate = TRUE,
                                  col = "#00000080", xlab = "retention time",
                                  ylab = "sample", xlim = range(rt),
@@ -711,13 +689,15 @@ adjustRtimePeakGroups <- function(object, param = PeakGroupsParam(),
                       dens_y[feat_range[1]:feat_range[2]] <- 0
                       feat_idx <- which(pks[, "rt"] >= dens$x[feat_range[1]] &
                                         pks[, "rt"] <= dens$x[feat_range[2]])
-                      tt <- table(sample_groups[pks[feat_idx, "sample"]])
+                      tt <- table(
+                          sample_groups[unique(pks[feat_idx, "sample"])])
                       if (!any(tt / sample_groups_table[names(tt)] >=
                                minFraction(param) & tt >= minSamples(param)))
                           next
                       rect(xleft = min(pks[feat_idx, "rt"]), ybottom = yl[1],
                            xright = max(pks[feat_idx, "rt"]), ytop = yl[2],
                            border = "#00000040", col = "#00000020")
+                      snum<- snum + 1L
                   }
         } else {
             ## Plot all features in the region.
@@ -741,58 +721,60 @@ adjustRtimePeakGroups <- function(object, param = PeakGroupsParam(),
 #'
 #' @description
 #'
-#' The \code{highlightChromPeaks} function adds chromatographic
+#' The `highlightChromPeaks()` function adds chromatographic
 #' peak definitions to an existing plot, such as one created by the
-#' \code{plot} method on a \code{\link{Chromatogram}} or
-#' \code{\link{MChromatograms}} object.
+#' `plot()` method on a [MSnbase::Chromatogram()] or
+#' [MSnbase::MChromatograms()] object.
 #'
-#' @param x For \code{highlightChromPeaks}: \code{XCMSnExp} object with the
+#' @param x For `highlightChromPeaks()`: `XCMSnExp` object with the
 #'     detected peaks.
 #'
-#' @param rt For \code{highlightChromPeaks}: \code{numeric(2)} with the
+#' @param rt For `highlightChromPeaks()`: `numeric(2)` with the
 #'     retention time range from which peaks should be extracted and plotted.
 #'
-#' @param mz \code{numeric(2)} with the mz range from which the peaks should
+#' @param mz `numeric(2)` with the mz range from which the peaks should
 #'     be extracted and plotted.
 #'
-#' @param peakIds \code{character} defining the IDs (i.e. rownames of the peak
-#'     in the \code{chromPeaks} table) of the chromatographic peaks to be
+#' @param peakIds `character` defining the IDs (i.e. rownames of the peak
+#'     in the `chromPeaks` table) of the chromatographic peaks to be
 #'     highlighted in a plot.
 #'
 #' @param border colors to be used to color the border of the rectangles/peaks.
-#'     Has to be equal to the number of samples in \code{x}.
+#'     Has to be equal to the number of samples in `x`.
 #'
-#' @param lwd \code{numeric(1)} defining the width of the line/border.
+#' @param lwd `numeric(1)` defining the width of the line/border.
 #'
-#' @param col For \code{highlightChromPeaks}: color to be used to fill the
-#'     rectangle (if \code{type = "rect"}) or the peak
-#'     (for \code{type = "polygon"}).
+#' @param col For `highlightChromPeaks()`: color to be used to fill the
+#'     rectangle (if `type = "rect"`) or the peak
+#'     (for `type = "polygon"`).
 #'
-#' @param type the plotting type. See \code{\link{plot}} in base grapics for
+#' @param type the plotting type. See `plot()` in base grapics for
 #'     more details.
-#'     For \code{highlightChromPeaks}: \code{character(1)} defining how the peak
-#'     should be highlighted: \code{type = "rect"} draws a rectangle
-#'     representing the peak definition, \code{type = "point"} indicates a
+#'     For `highlightChromPeaks()`: `character(1)` defining how the peak
+#'     should be highlighted: `type = "rect"` draws a rectangle
+#'     representing the peak definition, `type = "point"` indicates a
 #'     chromatographic peak with a single point at the position of the peak's
-#'     \code{"rt"} and \code{"maxo"} and \code{type = "polygon"} will highlight
-#'     the peak shape. For \code{type = "polygon"} the color of the border and
-#'     area can be defined with parameters \code{"border"} and \code{"col"},
+#'     `"rt"` and `"maxo"` and `type = "polygon"` will highlight
+#'     the peak shape. For `type = "polygon"` the color of the border and
+#'     area can be defined with parameters `"border"` and `"col"`,
 #'     respectively.
 #'
-#' @param whichPeaks \code{character(1)} specifying how peaks are called to be
-#'     located within the region defined by \code{mz} and \code{rt}. Can be
-#'     one of \code{"any"}, \code{"within"}, and \code{"apex_within"} for all
+#' @param whichPeaks `character(1)` specifying how peaks are called to be
+#'     located within the region defined by `mz` and `rt`. Can be
+#'     one of `"any"`, `"within"`, and `"apex_within"` for all
 #'     peaks that are even partially overlapping the region, peaks that are
 #'     completely within the region, and peaks for which the apex is within
-#'     the region. This parameter is passed to the \code{type} argument of the
-#'     \code{\link{chromPeaks}} function. See related documentation for more
+#'     the region. This parameter is passed to the `type` argument of the
+#'     [chromPeaks()] function. See related documentation for more
 #'     information and examples.
 #'
-#' @param ... additional parameters to the \code{\link{matplot}} or \code{plot}
+#' @param ... additional parameters to the `matplot()` or `plot()`
 #'     function.
 #'
 #' @author Johannes Rainer
 #'
+#' @md
+#'
 #' @examples
 #'
 #' ## Load a test data set with detected peaks
@@ -1596,7 +1578,8 @@ ms2_mspectrum_for_features <- function(x, expandRt = 0, expandMz = 0, ppm = 0,
         z <- peak_sp[idx[[i]]]
         if (any(lengths(z))) {
             z <- Spectra::concatenateSpectra(z)
-            z$feature_id <- fids[i]
+            z@backend@spectraData <- cbind(z@backend@spectraData,
+                                           DataFrame(feature_id = fids[i]))
             z@processing <- character()
             z
         }
@@ -1607,7 +1590,9 @@ ms2_mspectrum_for_features <- function(x, expandRt = 0, expandMz = 0, ppm = 0,
 
 #' @description
 #'
-#' \code{hasFilledChromPeaks}: whether filled-in peaks are present or not.
+#' `hasFilledChromPeaks()`: whether filled-in peaks are present or not.
+#'
+#' @md
 #'
 #' @rdname XCMSnExp-class
 setMethod("hasFilledChromPeaks", "XCMSnExp", function(object) {
@@ -1775,7 +1760,7 @@ setMethod("hasFilledChromPeaks", "XCMSnExp", function(object) {
                                      expand = 0, ppm = 0) {
     x[, min_col] <- x[, min_col] - expand - x[, min_col] * ppm / 1e6
     x[, max_col] <- x[, max_col] + expand + x[, max_col] * ppm / 1e6
-    reduced_ranges <- .reduce(x[, min_col], x[, max_col])
+    reduced_ranges <- do.call(cbind, reduce(x[, min_col], x[, max_col]))
     res <- vector("list", nrow(reduced_ranges))
     tolerance <- sqrt(.Machine$double.eps)
     for (i in seq_along(res)) {
